=== modified file 'libunity-2d-private/src/application.cpp'
--- libunity-2d-private/src/application.cpp	2012-02-16 11:24:19 +0000
+++ libunity-2d-private/src/application.cpp	2012-03-01 14:26:38 +0000
@@ -35,8 +35,7 @@
 #include "bamf-indicator.h"
 
 #include "dbusmenuimporter.h"
-
-#include <X11/X.h>
+#include "gobjectcallback.h"
 
 #include <gio/gio.h>
 
@@ -46,7 +45,9 @@
 
 // Qt
 #include <Qt>
+#include <QApplication>
 #include <QDebug>
+#include <QDesktopWidget>
 #include <QAction>
 #include <QDBusInterface>
 #include <QDBusReply>
@@ -56,12 +57,16 @@
 #include <QScopedPointer>
 #include <QX11Info>
 
+#include <X11/X.h>
+
 extern "C" {
 #include <gdk/gdk.h>
 #include <gdk/gdkx.h>
 #include <libsn/sn.h>
 }
 
+GOBJECT_CALLBACK0(geometryChangedCB, "onWindowGeometryChanged")
+
 const char* SHORTCUT_NICK_PROPERTY = "nick";
 
 Application::Application()
@@ -78,6 +83,11 @@
     m_launching_timer.setSingleShot(true);
     m_launching_timer.setInterval(8000);
     QObject::connect(&m_launching_timer, SIGNAL(timeout()), this, SLOT(onLaunchingTimeouted()));
+
+    // Accumulate geometry changes during 50 msec
+    m_geometryChangedTimer.setSingleShot(true);
+    m_geometryChangedTimer.setInterval(50);
+    connect(&m_geometryChangedTimer, SIGNAL(timeout()), this, SIGNAL(windowGeometryChanged()));
 }
 
 Application::Application(const Application& other)
@@ -91,6 +101,7 @@
 
 Application::~Application()
 {
+    disconnectWindowSignals();
 }
 
 bool
@@ -526,6 +537,34 @@
         WnckWindow* window = wnck_window_get(xids->at(i));
         g_signal_connect(G_OBJECT(window), "workspace-changed",
             G_CALLBACK(Application::onWindowWorkspaceChanged), this);
+        g_signal_connect(G_OBJECT(window), "geometry-changed", G_CALLBACK(geometryChangedCB), this);
+    }
+}
+
+void
+Application::disconnectWindowSignals()
+{
+    if (m_application == NULL || m_application->running() == false) {
+        return;
+    }
+
+    QScopedPointer<BamfUintList> xids(m_application->xids());
+    int size = xids->size();
+    if (size < 1) {
+        return;
+    }
+
+    for (int i = 0; i < size; ++i) {
+        WnckWindow *window = wnck_window_get(xids->at(i));
+        if (window == NULL) {
+            wnck_screen_force_update(wnck_screen_get_default());
+            window = wnck_window_get(xids->at(i));
+            if (window == NULL) {
+                continue;
+            }
+        }
+
+        g_signal_handlers_disconnect_by_func(window, gpointer(geometryChangedCB), this);
     }
 }
 
@@ -537,6 +576,7 @@
         WnckWindow* wnck_window = wnck_window_get(window->xid());
         g_signal_connect(G_OBJECT(wnck_window), "workspace-changed",
              G_CALLBACK(Application::onWindowWorkspaceChanged), this);
+        g_signal_connect(G_OBJECT(wnck_window), "geometry-changed", G_CALLBACK(geometryChangedCB), this);
     }
 }
 
@@ -986,6 +1026,44 @@
     return false;
 }
 
+bool
+Application::belongsToDifferentScreen(int screen)
+{
+    if (!m_application) {
+        return false;
+    }
+
+    if (QApplication::desktop()->screenCount() == 1) {
+        return false;
+    }
+
+    QScopedPointer<BamfUintList> xids(m_application->xids());
+    for (int i = 0; i < xids->size(); i++) {
+        /* When geting the wnck window, it's possible we get a NULL
+           return value because wnck hasn't updated its internal list yet,
+           so we need to force it once to be sure */
+        WnckWindow *window = wnck_window_get(xids->at(i));
+        if (window == NULL) {
+            wnck_screen_force_update(wnck_screen_get_default());
+            window = wnck_window_get(xids->at(i));
+            if (window == NULL) {
+                continue;
+            }
+        }
+
+        // Check the window screen
+        int x, y, width, height;
+        wnck_window_get_geometry(window, &x, &y, &width, &height);
+        const QRect windowRect(x, y, width, height);
+        const QPoint pos = windowRect.center();
+        if (QApplication::desktop()->screenNumber(pos) == screen) {
+            return false;
+        }
+    }
+
+    return true;
+}
+
 void
 Application::onIndicatorMenuUpdated()
 {
@@ -1110,6 +1188,13 @@
 }
 
 void
+Application::onWindowGeometryChanged()
+{
+    if (!m_geometryChangedTimer.isActive())
+      m_geometryChangedTimer.start();
+}
+
+void
 Application::onDragEnter(DeclarativeDragDropEvent* event)
 {
     QList<QUrl> urls = validateUrisForLaunch(event->mimeData());

=== modified file 'libunity-2d-private/src/application.h'
--- libunity-2d-private/src/application.h	2012-02-16 11:24:19 +0000
+++ libunity-2d-private/src/application.h	2012-03-01 14:26:38 +0000
@@ -105,7 +105,9 @@
 
     Q_INVOKABLE virtual void createMenuActions();
     Q_INVOKABLE virtual bool belongsToDifferentWorkspace();
+    Q_INVOKABLE virtual bool belongsToDifferentScreen(int screen);
     Q_INVOKABLE void connectWindowSignals();
+    void disconnectWindowSignals();
 
     void updateOverlaysState(const QString& sender, const QMap<QString, QVariant>& properties);
 
@@ -151,6 +153,8 @@
     void onDragEnter(DeclarativeDragDropEvent*);
     void onDrop(DeclarativeDragDropEvent*);
 
+    void onWindowGeometryChanged();
+
 private:
     QPointer<BamfApplication> m_application;
     QFileSystemWatcher *m_desktopFileWatcher;
@@ -188,6 +192,7 @@
     QDBusServiceWatcher* m_dynamicQuicklistServiceWatcher;
     void setDynamicQuicklistImporter(const QString& service);
     IndicatorDesktopShortcutsPointer m_staticShortcuts;
+    QTimer m_geometryChangedTimer;
 };
 
 Q_DECLARE_METATYPE(Application*)

=== modified file 'libunity-2d-private/src/launcheritem.cpp'
--- libunity-2d-private/src/launcheritem.cpp	2012-02-06 12:07:47 +0000
+++ libunity-2d-private/src/launcheritem.cpp	2012-03-01 14:26:38 +0000
@@ -93,6 +93,12 @@
 }
 
 bool
+LauncherItem::belongsToDifferentScreen(int)
+{
+    return false;
+}
+
+bool
 LauncherItem::progressBarVisible() const
 {
     return false;

=== modified file 'libunity-2d-private/src/launcheritem.h'
--- libunity-2d-private/src/launcheritem.h	2012-02-06 12:07:47 +0000
+++ libunity-2d-private/src/launcheritem.h	2012-03-01 14:26:38 +0000
@@ -80,6 +80,7 @@
     Q_INVOKABLE virtual void createMenuActions() = 0;
     Q_INVOKABLE virtual void launchNewInstance();
     Q_INVOKABLE virtual bool belongsToDifferentWorkspace();
+    Q_INVOKABLE virtual bool belongsToDifferentScreen(int screen);
 
 protected:
     LauncherContextualMenu* m_menu;
@@ -102,6 +103,7 @@
     void emblemVisibleChanged(bool);
     void emblemChanged(QString);
     void windowWorkspaceChanged();
+    void windowGeometryChanged();
 
 public Q_SLOTS:
     /* Default implementation of drag’n’drop handling, should be overridden in

